__all__ = (
    "TOMLDecodeError",
    "TOMLEncodeError",
    "__version__",
    "dump",
    "dumps",
    "load",
    "loads",
)

from collections.abc import Callable
from pathlib import Path
from typing import Any, BinaryIO, TextIO

from ._toml_rs import (
    _dumps,
    _loads,
    _version,
)

__version__: str = _version


def load(fp: BinaryIO, /, *, parse_float: Callable[[str], Any] = float) -> dict[str, Any]:
    _bytes = fp.read()
    try:
        _str = _bytes.decode()
    except AttributeError:
        msg = "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`"
        raise TypeError(msg) from None
    return loads(_str, parse_float=parse_float)


def loads(s: str, /, *, parse_float: Callable[[str], Any] = float) -> dict[str, Any]:
    if not isinstance(s, str):
        raise TypeError(f"Expected str object, not '{type(s).__name__}'")
    return _loads(s, parse_float=parse_float)


def dump(
        obj: Any,
        /,
        file: str | Path | TextIO,
        inline_tables: set[str] | None = None,
        *,
        pretty: bool = False,
) -> int:
    _str = _dumps(obj, inline_tables=inline_tables, pretty=pretty)
    if isinstance(file, str):
        file = Path(file)
    if isinstance(file, Path):
        return file.write_text(_str, encoding="utf-8")
    else:
        return file.write(_str)


def dumps(
        obj: Any,
        /,
        inline_tables: set[str] | None = None,
        *,
        pretty: bool = False,
) -> str:
    return _dumps(obj, inline_tables=inline_tables, pretty=pretty)


class TOMLDecodeError(ValueError):
    def __init__(self, msg: str, doc: str, pos: int, *args: Any):
        msg = msg.rstrip()
        super().__init__(msg)
        lineno = doc.count("\n", 0, pos) + 1
        if lineno == 1:
            colno = pos + 1
        else:
            colno = pos - doc.rindex("\n", 0, pos)
        self.msg = msg
        self.doc = doc
        self.pos = pos
        self.colno = colno
        self.lineno = lineno


class TOMLEncodeError(TypeError):
    def __init__(self, msg: str, *args: Any):
        msg = msg.rstrip()
        super().__init__(msg)
        self.msg = msg



import json
from dataclasses import dataclass
from pathlib import Path

import pytest
import toml_rs as tomllib

from tests import _init_only, tests_path
from .burntsushi import convert, normalize


@dataclass(**_init_only)
class MissingFile:
    path: Path


DATA_DIR = tests_path / "data"


# Test files were taken from this commit:
# https://github.com/toml-lang/toml-test/commit/1d35870ef6783d86366ba55d7df703f3f60b3b55
def read_toml_files_file():
    lines = (
        (DATA_DIR / "files-toml-1.0.0")
        .read_text(encoding="utf-8", errors="ignore")
        .splitlines()
    )
    return (
        tuple(
            DATA_DIR / line.strip()
            for line in lines
            if line.strip().endswith(".toml") and line.strip().startswith("valid/")
        ),
        tuple(
            DATA_DIR / line.strip()
            for line in lines
            if line.strip().endswith(".toml") and line.strip().startswith("invalid/")
        ),
    )


VALID_FILES, INVALID_FILES = read_toml_files_file()
assert VALID_FILES, "Valid TOML test files not found"
assert INVALID_FILES, "Invalid TOML test files not found"

VALID_FILES_EXPECTED = tuple(
    json.loads((p.with_suffix(".json")).read_text()) for p in VALID_FILES
)


@pytest.mark.parametrize("invalid", INVALID_FILES, ids=lambda p: p.stem)
def test_invalid(invalid):
    toml_bytes = invalid.read_bytes()
    try:
        toml_str = toml_bytes.decode()
    except UnicodeDecodeError:
        # Some BurntSushi tests are not valid UTF-8. Skip those.
        pytest.skip(f"Invalid UTF-8: {invalid}")
    with pytest.raises(tomllib.TOMLDecodeError):
        tomllib.loads(toml_str)


VALID_PAIRS = list(zip(VALID_FILES, VALID_FILES_EXPECTED, strict=False))


@pytest.mark.parametrize(
    ("valid", "expected"),
    VALID_PAIRS,
    ids=[p[0].stem for p in VALID_PAIRS],
)
def test_valid(valid, expected):
    toml_str = valid.read_bytes().decode("utf-8")
    try:
        toml_str.encode("ascii")
    except UnicodeEncodeError:
        pytest.skip(f"Skipping Unicode content test: {valid.name}")
    actual = tomllib.loads(toml_str)
    actual = convert(actual)
    expected_normalized = normalize(expected)
    assert actual == expected_normalized
    # Ensure that parsed toml's can be serialized back without error
    tomllib.dumps(actual)



import tomllib
from pathlib import Path

import pytest
import toml_rs

from tests import TOML
from .burntsushi import convert, normalize
from .test_data import VALID_PAIRS


def test_toml():
    toml_str = TOML.read_text(encoding="utf-8")
    assert tomllib.loads(toml_str) == toml_rs.loads(toml_str)


@pytest.mark.parametrize("lib", [tomllib, toml_rs])
def test_text_mode_typeerror(lib):
    err_msg = "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`"
    with Path(TOML).open(encoding="utf-8") as f, pytest.raises(TypeError) as exc:
        lib.load(f)
    assert err_msg in str(exc.value)


@pytest.mark.parametrize(
    ("valid", "expected"),
    VALID_PAIRS,
    ids=[p[0].stem for p in VALID_PAIRS],
)
def test_tomllib_vs_tomlrs(valid, expected):
    toml_str = valid.read_bytes().decode("utf-8")
    try:
        toml_str.encode("ascii")
    except UnicodeEncodeError:
        pytest.skip(f"Skipping Unicode content test: {valid.name}")
    tomllib_ = normalize(convert(tomllib.loads(toml_str)))
    toml_rs_ = normalize(convert(toml_rs.loads(toml_str)))

    assert tomllib_ == toml_rs_, (
        f"Mismatch between tomllib and toml_rs for {valid.name}"
    )
